package com.greymerk.roguelike.editor.factories;

import com.greymerk.roguelike.editor.Coord;
import com.greymerk.roguelike.editor.IBlockFactory;
import com.greymerk.roguelike.editor.IWorldEditor;
import com.greymerk.roguelike.editor.blocks.BlockType;

import net.minecraft.util.math.random.Random;

/**
 * This BlockFactory serves the special purpose of
 * creating solid floor unless it's being placed in
 * mid air, in which case there will be holes.
 * 
 * This type is automatically generated by the BlockSet
 * class and thus never needs to be directly instantiated.
 * 
 * @author brian
 */
public class BlockFloor extends BlockBase implements IBlockFactory{

	private IBlockFactory floor;
	private IBlockFactory bridge;
	
	public BlockFloor(IBlockFactory floor) {
		this.floor = floor;
		BlockWeightedRandom bridge = new BlockWeightedRandom();
		bridge.addBlock(floor, 10);
		bridge.addBlock(BlockType.get(BlockType.AIR), 1);
		this.bridge = bridge;
	}
	
	@Override
	public boolean set(IWorldEditor editor, Random rand, Coord pos, boolean fillAir, boolean replaceSolid) {
		if(!fillAir && editor.isAir(pos)) return false;
		if(!replaceSolid && editor.isSolid(pos)) return false;
		
		floor.set(editor, rand, pos, false, true);
		bridge.set(editor, rand, pos, true, false);
		return true;
	}

}
